@@ -7,7 +7,8 @@
 
 #define CH_BUF_SIZE 128
 
-int the_test_phase=0;
+int the_test_phase = 0;
+int memory_freed = 1; /* when set to 0, freeglobaldata() should be called on re-entering calls to startDDE - only applicable to interupted calculations */
 
 /*===========================================================================*/
 /* The function lang5 is part of the C interface for R after version 2.11    */
@@ -26,7 +27,8 @@ SEXP lang5(SEXP s, SEXP t, SEXP u, SEXP v, SEXP w)
 void PBSerror(char *str)
 {
 	error(str);
-	exit(-1);
+	/* exit(-1); */
+	return(-1);
 }
 
 /*===========================================================================*/
@@ -42,19 +44,19 @@ void output(double *s,double t)
 	  and [1..(no_var+1)] are reserved for s[0..no_var] vars
 	  */
 	int i;
-	static double *dummy_var=NULL;
-	if( dummy_var == NULL )
-		dummy_var = malloc( data.no_var*sizeof(double) );
 	data.vals[0][data.vals_ind] = t;
 	for( i = 0; i < data.no_var; i++ )
 		data.vals[i+1][data.vals_ind] = s[i];
 	
 	/*ACB hack - call grad to pull out any other data*/
-	if( data.no_otherVars > 0 )
-		grad(dummy_var,s,NULL,t);
-	
-	for( i = 0; i < data.no_otherVars; i++ )
-		data.vals[1+data.no_var+i][data.vals_ind] = data.tmp_other_vals[i];
+	/* without this call, the other values (returned by grad) won't be calculated exactly at t, but rather at t+/-delta (where delta < step size) which is used during the integration */
+	if( data.no_otherVars > 0 ) {
+		grad(NULL,s,NULL,t);	/* cause a calc exactly at t */
+
+		/* then save the extra variables retuend in the second component of the R grad func */
+		for( i = 0; i < data.no_otherVars; i++ )
+			data.vals[1+data.no_var+i][data.vals_ind] = data.tmp_other_vals[i];
+	}
 	
 	data.vals_ind++;
 	
@@ -70,13 +72,18 @@ void output(double *s,double t)
 
 /*===========================================================================*/
 /* cont is zero for fresh run, and 1 for continuation */
-void numerics(double *c,int cont)
+void numerics(double *c,int cont, int clear)
 { 
 	static double *s;
 	double t0, t1, dt, *otimes; /* bjc 2007-05-08*/
 	int ns, nsw, nhv, nlag, reset=1, fixstep=0, no_otimes; /* bjc 2007-05-08*/
 	static int first=1;
 	long hbsize;
+
+	if(clear && first==0) { /* Bobby */
+	  free(s); s = NULL; first = 1; return;
+	} else if(clear) return;
+
 	ns=data.no_var;
 	nsw=data.nsw;
 	nhv=data.nhv;
@@ -87,18 +94,19 @@ void numerics(double *c,int cont)
 	hbsize=data.hbsize;
 	otimes=data.otimes;
 	no_otimes=data.no_otimes; /* bjc 2007-05-08*/
-  
+
 	if (cont) {
 		reset=0;
 	} else {
 		if (!first) {
 			free(s);
-			first=0;
+			/* first=0; */ /* Bobby */
   		}
 		s=(double *)calloc(data.no_var,sizeof(double));
+		first = 0; /* bobby */
 		ddeinitstate(s,c,t0);
 	}
-	dde(s,c,t0,t1,&dt,data.tol,otimes,no_otimes,ns,nsw,nhv,hbsize,nlag,reset,fixstep); /* bjc 2007-05-08*/
+	dde(s,c,t0,t1,&dt,data.tol,otimes,no_otimes,ns,nsw,nhv,hbsize,nlag,reset,fixstep,0); /* bjc 2007-05-08*/
 	data.dt=dt;
 }
 
@@ -161,6 +169,17 @@ void freeglobaldata()
 		free(data.tmp_other_vals);
 		data.tmp_other_vals=NULL;
 	}
+	/* fprintf(stdout, "freed global data\n"); */
+
+	/* Bobby */
+	/* necessary to free the memory allocated to static pointers
+	   within the following functions */
+	istep(NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,0,0,0,0,NULL,1);
+	inithisbuff(0, 0, 0, 1);
+	numerics(NULL, 0, 1);
+	dde(NULL,NULL,0,0,NULL,0,NULL,0,0,0,0,0,0,1,0,1);
+	rk23(NULL,NULL,NULL,NULL,NULL,NULL,0,0,0,1);
+	updatehistory(NULL,NULL,NULL,0,1);
 }
 
 int testFunc(int no_var, double *test_vars, double t, SEXP *names, PROTECT_INDEX *names_ipx)
@@ -305,6 +324,12 @@ int testMapFunc(int no_var, double *test_vars, double t, int switch_num)
 /*===========================================================================*/
 SEXP startDDE(SEXP gradFunc, SEXP switchFunc, SEXP mapFunc, SEXP env, SEXP yinit, SEXP parms, SEXP settings, SEXP outtimes)
 {
+	/* free memory on successive calls to startDDE (to prevent leaked memory when R interupts this routine) */
+	if( memory_freed == 0 ) {
+		memory_freed = 1;
+		freeglobaldata();
+	}
+
 	SEXP list, vect, extra_names, yinit_names, names;
 	PROTECT_INDEX extra_names_ipx;
 	double *p, *otimes; /* bjc 2007-05-08*/
@@ -393,9 +418,10 @@ SEXP startDDE(SEXP gradFunc, SEXP switchFunc, SEXP mapFunc, SEXP env, SEXP yinit
 	}
 	
 	setupglobaldata(LENGTH(yinit), no_otherVar, no_switch, NUMERIC_POINTER(settings), otimes, no_otimes); /* bjc 2007-05-08*/
+	memory_freed = 0;
 	
 	/* preform dde calculations */
-	numerics(NUMERIC_POINTER(yinit), 0);
+	numerics(NUMERIC_POINTER(yinit), 0, 0);
 	
 	/* create list which will be base of polyset data.frame */
 	PROTECT(list=allocVector(VECSXP, data.no_var+data.no_otherVars+1));
@@ -425,7 +451,8 @@ SEXP startDDE(SEXP gradFunc, SEXP switchFunc, SEXP mapFunc, SEXP env, SEXP yinit
 	setAttrib(list, R_NamesSymbol, names);
 
 	UNPROTECT( 3 );
-  freeglobaldata();
+	freeglobaldata();
+	memory_freed = 1;
 	return list;
 }
 
